@@ -1,3 +1,22 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) 2008 Red Hat, Inc.  All rights reserved.
+**  All rights reserved.
+**
+**  Author: Fabio M. Di Nitto <fdinitto@redhat.com>
+**
+**  Original design by:
+**  Joel Becker <Joel.Becker@oracle.com>
+**  Fabio M. Di Nitto <fdinitto@redhat.com>
+**
+**  This copyrighted material is made available to anyone wishing to use,
+**  modify, copy, or redistribute it subject to the terms and conditions
+**  of the GNU General Public License v.2.
+**
+*******************************************************************************
+******************************************************************************/
+
 #include <stdio.h>
 #include <time.h>
 #include <stdlib.h>
@@ -466,8 +485,13 @@ static int sysfs_is_dev(char *path, int *maj, int *min)
 	if (!lstat(newpath, &sb)) {
 		f = fopen(newpath, "r");
 		if (f) {
-			fscanf(f, "%d:%d", maj, min);
+			int err;
+
+			err = fscanf(f, "%d:%d", maj, min);
 			fclose(f);
+			if ((err == EOF) || (err != 2))
+				return -1;
+
 			return 1;
 		} else
 			return -1;
@@ -494,8 +518,12 @@ static int sysfs_is_removable(char *path)
 	if (!lstat(newpath, &sb)) {
 		f = fopen(newpath, "r");
 		if (f) {
-			fscanf(f, "%d\n", &i);
+			int err;
+
+			err = fscanf(f, "%d\n", &i);
 			fclose(f);
+			if ((err == EOF) || (err != 1))
+				i = -1;
 		}
 	}
 	return i;
@@ -576,9 +604,14 @@ static int sysfs_is_disk(char *path)
       found:
 	f = fopen(newpath, "r");
 	if (f) {
-		fscanf(f, "%d\n", &i);
+		int err;
+
+		err = fscanf(f, "%d\n", &i);
 		fclose(f);
 
+		if ((err == EOF) || (err != 1))
+			return 0;
+
 		switch (i) {
 		case 0x0:	/* scsi type_disk */
 		case 0xe:	/* found on ide disks from old kernels.. */
