@@ -1,14 +1,18 @@
 #!/usr/bin/python
 
+##
+## Copyright (C) 2008 Red Hat, Inc. All Rights Reserved.
+##
+#####
 import sys, getopt, time, os
 import pexpect, re
 import telnetlib
 
 ## do not add code here.
 #BEGIN_VERSION_GENERATION
-RELEASE_VERSION = "New fence lib agent - test release on steroids"
-REDHAT_COPYRIGHT = ""
-BUILD_DATE = "March, 2008"
+RELEASE_VERSION="New fence lib agent - test release on steroids"
+REDHAT_COPYRIGHT=""
+BUILD_DATE="March, 2008"
 #END_VERSION_GENERATION
 
 POWER_TIMEOUT = 20
@@ -24,11 +28,6 @@ EC_CONNECTION_LOST = 4
 EC_TIMED_OUT       = 5
 EC_WAITING_ON      = 6
 EC_WAITING_OFF     = 7
-EC_STATUS          = 8
-
-TELNET_PATH = "/usr/bin/telnet"
-SSH_PATH    = "/usr/bin/ssh"
-SSL_PATH    = "@FENCEAGENTSLIBDIR@/telnet_ssl"
 
 all_opt = {
 	"help"    : {
@@ -71,22 +70,14 @@ all_opt = {
 		"getopt" : "",
 		"help" : "",
 		"order" : 1 },
-	"no_password" : {
-		"getopt" : "",
-		"help" : "",
-		"order" : 1 },
 	"passwd" : {
 		"getopt" : "p:",
-		"help" : "-p <password>  Login password or passphrase",
+		"help" : "-p <password>  Login password",
 		"order" : 1 },
 	"passwd_script" : {
 		"getopt" : "S:",
 		"help" : "-S <script>    Script to run to retrieve password",
 		"order" : 1 },
-	"identity_file" : {
-		"getopt" : "k:",
-		"help" : "-k <filename>  Identity file (private key) for ssh ",
-		"order" : 1 },
 	"module_name" : {
 		"getopt" : "m:",
 		"help" : "-m <module>    DRAC/MC module name",
@@ -107,47 +98,15 @@ all_opt = {
 		"getopt" : "x",
 		"help" : "-x             Use ssh connection",
 		"order" : 1 },
-	"ssl" : {
-		"getopt" : "z",
-		"help" : "-z             Use ssl connection",
-		"order" : 1 },
 	"port" : {
 		"getopt" : "n:",
-		"help" : "-n <id>        Physical plug number on device or name of virtual machine",
-		"order" : 1 },
-	"switch" : {
-		"getopt" : "s:",
-		"help" : "-s <id>        Physical switch number on device",
-		"order" : 1 },
-	"partition" : {
-		"getopt" : "n:",
-		"help" : "-n <id>        Name of the partition",
-		"order" : 1 },
-	"managed" : {
-		"getopt" : "s:",
-		"help" : "-s <id>        Name of the managed system",
+		"help" : "-n <id>        Physical plug number on device",
 		"order" : 1 },
 	"test" : {
 		"getopt" : "T",
 		"help" : "",
 		"order" : 1,
-		"obsolete" : "use -o status instead" },
-	"vmipaddr" : {
-		"getopt" : "A:",
-		"help" : "-A <ip>        IP address or hostname of managed VMware ESX (default localhost)",
-		"order" : 2 },
-	"vmlogin" : {
-		"getopt" : "L:",
-		"help" : "-L <name>      VMware ESX management login name",
-		"order" : 2 },
-	"vmpasswd" : {
-		"getopt" : "P:",
-		"help" : "-P <password>  VMware ESX management login password",
-		"order" : 2 },
-	"vmpasswd_script" : {
-		"getopt" : "B:",
-		"help" : "-B <script>    Script to run to retrieve VMware ESX management password",
-		"order" : 2 }
+		"obsolete" : "use -o status instead" }
 }
 
 class fspawn(pexpect.spawn):
@@ -157,10 +116,10 @@ class fspawn(pexpect.spawn):
 			options["debug_fh"].write(self.before + self.after)
 		return result
 
-def version(command, release, build_date, copyright_notice):
-	print command, " ", release, " ", build_date
-	if len(copyright_notice) > 0:
-		print copyright_notice
+def version(command, release, build_date, copyright):
+	print command, " ", release, " ", build_date;
+	if len(copyright) > 0:
+		print copyright
 
 def fail_usage(message = ""):
 	if len(message) > 0:
@@ -174,8 +133,7 @@ def fail(error_code):
 		EC_CONNECTION_LOST : "Connection lost",
 		EC_TIMED_OUT : "Connection timed out",
 		EC_WAITING_ON : "Failed: Timed out waiting to power ON",
-		EC_WAITING_OFF : "Failed: Timed out waiting to power OFF",
-		EC_STATUS : "Failed: Unable to obtain correct plug status"
+		EC_WAITING_OFF : "Failed: Timed out waiting to power OFF"
 	}[error_code] + "\n"
 	sys.stderr.write(message)
 	sys.exit(error_code)
@@ -200,8 +158,7 @@ def process_input(avail_opt):
 	##
 	## Set standard environment
 	#####
-	os.putenv("LANG", "C")
-	os.putenv("LC_ALL", "C")
+	os.unsetenv("LANG")
 
 	##
 	## Prepare list of options for getopt
@@ -211,7 +168,7 @@ def process_input(avail_opt):
 		if all_opt.has_key(k):
 			getopt_string += all_opt[k]["getopt"]
 		else:
-			fail_usage("Parse error: unknown option '"+k+"'")
+			fail_usage("Parse error: unknown option '"+k+"'");
 
 	##
 	## Read options from command line or standard input
@@ -236,8 +193,7 @@ def process_input(avail_opt):
 		name = ""
 		for line in sys.stdin.readlines():
 			line = line.strip()
-			if ((line.startswith("#")) or (len(line) == 0)):
-				continue
+			if ((line.startswith("#")) or (len(line) == 0)): pass
 
 			(name, value) = (line + "=").split("=", 1)
 			value = value[:-1]
@@ -289,9 +245,6 @@ def check_input(device_opt, opt):
 	if 0 == options.has_key("-o"):
 		options["-o"] = "reboot"
 
-	# Convert action to lowercase
-	options["-o"]=options["-o"].lower()
-
 	if 0 == ["on", "off", "reboot", "status"].count(options["-o"].lower()):
 		fail_usage("Failed: Unrecognised action '" + options["-o"] + "'")
 
@@ -301,22 +254,13 @@ def check_input(device_opt, opt):
 	if 0 == options.has_key("-a"):
 		fail_usage("Failed: You have to enter fence address")
 
-	if (device_opt.count("no_password") == 0):
-		if 0 == device_opt.count("identity_file"):
-			if 0 == (options.has_key("-p") or options.has_key("-S")):
-				fail_usage("Failed: You have to enter password or password script")
-			else: 
-				if 0 == (options.has_key("-p") or options.has_key("-S") or options.has_key("-k")):
-					fail_usage("Failed: You have to enter password, password script or identity file")
+	if 0 == (options.has_key("-p") or options.has_key("-S")):
+		fail_usage("Failed: You have to enter password or password script")
 
-	if 0 == options.has_key("-x") and 1 == options.has_key("-k"):
-		fail_usage("Failed: You have to use identity file together with ssh connection (-x)")
+	if 1 == (options.has_key("-p") and options.has_key("-S")):
+		fail_usage("Failed: You have to enter password or password script")
 
-	if 1 == options.has_key("-k"):
-		if 0 == os.path.isfile(options["-k"]):
-			fail_usage("Failed: Identity file " + options["-k"] + " does not exist")
-
-	if (0 == options.has_key("-n")) and (device_opt.count("port")):
+	if (0 == options.has_key("-n")) and (device_opt.count("plug_no")):
 		fail_usage("Failed: You have to enter plug number")
 
 	if options.has_key("-S"):
@@ -331,24 +275,10 @@ def check_input(device_opt, opt):
 	if options.has_key("-v") and options.has_key("debug_fh") == 0:
 		options["debug_fh"] = sys.stderr
 
-	## VMware
-	#######
-	if options.has_key("-B"):
-		options["-P"] = os.popen(options["-B"]).read().rstrip()
-
-	if (device_opt.count("vmlogin") and (not options.has_key("-L"))):
-		fail_usage("Failed: You have to set login name for VMware ESX management console")
-
-	if (options.has_key("-L") and (not (options.has_key("-P") or options.has_key("-C")))):
-		fail_usage("Failed: You have to enter password or password script for VMware ESX management console")
-
-	if (options.has_key("-L") and (not (options.has_key("-n")))):
-		fail_usage("Failed: You have to enter virtual machine name")
-		
 	return options
 	
 def wait_power_status(tn, options, get_power_fn):
-	for dummy in xrange(POWER_TIMEOUT):
+	for x in range(POWER_TIMEOUT):
 		if get_power_fn(tn, options) != options["-o"]:
 			time.sleep(1)
 		else:
@@ -358,9 +288,6 @@ def wait_power_status(tn, options, get_power_fn):
 def fence_action(tn, options, set_power_fn, get_power_fn):
 	status = get_power_fn(tn, options)
 
-	if status != "on" and status != "off":  
-		fail(EC_STATUS)
-
 	if options["-o"] == "on":
 		if status == "on":
 			print "Success: Already ON"
@@ -388,7 +315,7 @@ def fence_action(tn, options, set_power_fn, get_power_fn):
 		options["-o"] = "on"
 		set_power_fn(tn, options)
 		if wait_power_status(tn, options, get_power_fn) == 0:
-			sys.stderr.write('Timed out waiting to power ON\n')
+			fail(EC_WAITING_ON)
 		print "Success: Rebooted"
 	elif options["-o"] == "status":
 		print "Status: " + status.upper()
@@ -398,50 +325,16 @@ def fence_login(options):
 		re_login = re.compile("(login: )|(Login Name:  )|(username: )|(User Name :)", re.IGNORECASE)
 		re_pass  = re.compile("password", re.IGNORECASE)
 
-		if options.has_key("-z"):
-			command = '%s %s %s' % (SSL_PATH, options["-a"], "443")
-			conn = fspawn(command)
-		elif options.has_key("-x") and 0 == options.has_key("-k"):
-			command = '%s %s@%s' % (SSH_PATH, options["-l"], options["-a"])
-			if options.has_key("ssh_options"):
-				command += ' ' + options["ssh_options"]
-			conn = fspawn(command)
-
-			if options.has_key("telnet_over_ssh"):
-				#This is for stupid ssh servers (like ALOM) which behave more like telnet (ignore name and display login prompt)
-				result = conn.log_expect(options, [ re_login, "Are you sure you want to continue connecting (yes/no)?" ], LOGIN_TIMEOUT)
-				if result == 1:
-					conn.sendline("yes") # Host identity confirm
-					conn.log_expect(options, re_login, LOGIN_TIMEOUT)
-
-				conn.sendline(options["-l"])
-				conn.log_expect(options, re_pass, LOGIN_TIMEOUT)
-			else:
-				result = conn.log_expect(options, [ "ssword:", "Are you sure you want to continue connecting (yes/no)?" ], LOGIN_TIMEOUT)
-				if result == 1:
-					conn.sendline("yes")
-					conn.log_expect(options, "ssword:", LOGIN_TIMEOUT)
-
-			conn.sendline(options["-p"])
-			conn.log_expect(options, options["-c"], LOGIN_TIMEOUT)
-		elif options.has_key("-x") and 1 == options.has_key("-k"):
-			command = '%s %s@%s -i %s' % (SSH_PATH, options["-l"], options["-a"], options["-k"])
-			if options.has_key("ssh_options"):
-				command += ' ' + options["ssh_options"]
-			conn = fspawn(command)
-
-			result = conn.log_expect(options, [ options["-c"], "Are you sure you want to continue connecting (yes/no)?", "Enter passphrase for key '"+options["-k"]+"':" ], LOGIN_TIMEOUT)
+		if options.has_key("-x"):
+			conn = fspawn ('ssh ' + options["-l"] + "@" + options["-a"])
+			result = conn.log_expect(options, [ "ssword: ", "Are you sure you want to continue connecting (yes/no)?" ], LOGIN_TIMEOUT)
 			if result == 1:
 				conn.sendline("yes")
-				conn.log_expect(options, [ options["-c"], "Enter passphrase for key '"+options["-k"]+"':"] , LOGIN_TIMEOUT)
-			if result != 0:
-				if options.has_key("-p"):
-					conn.sendline(options["-p"])
-					conn.log_expect(options, options["-c"], LOGIN_TIMEOUT)
-				else:
-					fail_usage("Failed: You have to enter passphrase (-p) for identity file")
+				conn.log_expect(options, "ssword: ", SHELL_TIMEOUT)
+			conn.sendline(options["-p"])
+			conn.log_expect(options, options["-c"], SHELL_TIMEOUT)
 		else:
-			conn = fspawn('%s %s' % (TELNET_PATH, options["-a"]))
+			conn = fspawn ('telnet ' + options["-a"])
 			conn.log_expect(options, re_login, LOGIN_TIMEOUT)
 			conn.send(options["-l"]+"\r\n")
 			conn.log_expect(options, re_pass, SHELL_TIMEOUT)
