@@ -263,50 +263,60 @@ static int do_external(char *name, char *extra, int extra_len)
 	return rv;
 }
 
+static void init_header(struct fenced_header *h, int cmd, int result,
+			int extra_len)
+{
+	memset(h, 0, sizeof(struct fenced_header));
+
+	h->magic = FENCED_MAGIC;
+	h->version = FENCED_VERSION;
+	h->len = sizeof(struct fenced_header) + extra_len;
+	h->command = cmd;
+	h->data = result;
+}
+
 /* combines a header and the data and sends it back to the client in
    a single do_write() call */
 
-static void do_reply(int f, int cmd, int result, char *buf, int len)
+static void do_reply(int f, int cmd, int result, char *buf, int buflen)
 {
-	struct fenced_header *rh;
 	char *reply;
 	int reply_len;
 
-	reply_len = sizeof(struct fenced_header) + len;
+	reply_len = sizeof(struct fenced_header) + buflen;
 	reply = malloc(reply_len);
 	if (!reply)
 		return;
 	memset(reply, 0, reply_len);
 
-	rh = (struct fenced_header *)reply;
+	init_header((struct fenced_header *)reply, cmd, result, buflen);
 
-	rh->magic = FENCED_MAGIC;
-	rh->version = FENCED_VERSION;
-	rh->len = reply_len;
-	rh->command = cmd;
-	rh->data = result;
-
-	if (buf)
-		memcpy(reply + sizeof(struct fenced_header), buf, len);
+	if (buf && buflen)
+		memcpy(reply + sizeof(struct fenced_header), buf, buflen);
 
 	do_write(f, reply, reply_len);
 
 	free(reply);
 }
 
-/* dump can do 1, 2, or 3 separate writes:
-   first write is the header,
-   second write is the tail of log,
-   third is the head of the log */
-
 static void query_dump_debug(int f)
 {
+	struct fenced_header h;
+	int extra_len;
 	int len;
 
-	do_reply(f, FENCED_CMD_DUMP_DEBUG, 0, NULL, 0);
+	/* in the case of dump_wrap, extra_len will go in two writes,
+	   first the log tail, then the log head */
+	if (dump_wrap)
+		extra_len = FENCED_DUMP_SIZE;
+	else
+		extra_len = dump_point;
+
+	init_header(&h, FENCED_CMD_DUMP_DEBUG, 0, extra_len);
+	do_write(f, &h, sizeof(h));
 
 	if (dump_wrap) {
-		len = DUMP_SIZE - dump_point;
+		len = FENCED_DUMP_SIZE - dump_point;
 		do_write(f, dump_buf + dump_point, len);
 		len = dump_point;
 	} else
@@ -358,40 +368,47 @@ static void query_domain_info(int f)
 	do_reply(f, FENCED_CMD_DOMAIN_INFO, rv, (char *)&domain, sizeof(domain));
 }
 
-static void query_domain_members(int f, int max)
+static void query_domain_nodes(int f, int option, int max)
 {
 	struct fd *fd;
-	int member_count = 0;
-	struct fenced_node *members = NULL;
-	int rv;
+	int node_count = 0;
+	struct fenced_node *nodes = NULL;
+	int rv, result;
 
 	fd = find_fd("default");
 	if (!fd) {
-		rv = -ENOENT;
+		result = -ENOENT;
+		node_count = 0;
 		goto out;
 	}
 
 	if (group_mode == GROUP_LIBGROUP)
-		rv = set_domain_members(fd, &member_count, &members);
+		rv = set_domain_nodes_group(fd, option, &node_count, &nodes);
 	else
-		rv = set_domain_members(fd, &member_count, &members);
+		rv = set_domain_nodes(fd, option, &node_count, &nodes);
 
 	if (rv < 0) {
-		member_count = 0;
+		result = rv;
+		node_count = 0;
 		goto out;
 	}
-	if (member_count > max) {
-		rv = -E2BIG;
-		member_count = max;
+
+	/* node_count is the number of structs copied/returned; the caller's
+	   max may be less than that, in which case we copy as many as they
+	   asked for and return -E2BIG */
+
+	if (node_count > max) {
+		result = -E2BIG;
+		node_count = max;
 	} else {
-		rv = member_count;
+		result = node_count;
 	}
  out:
-	do_reply(f, FENCED_CMD_DOMAIN_MEMBERS, rv,
-	         (char *)members, member_count * sizeof(struct fenced_node));
+	do_reply(f, FENCED_CMD_DOMAIN_NODES, result,
+	         (char *)nodes, node_count * sizeof(struct fenced_node));
 
-	if (members)
-		free(members);
+	if (nodes)
+		free(nodes);
 }
 
 static void process_connection(int ci)
@@ -445,7 +462,7 @@ static void process_connection(int ci)
 	case FENCED_CMD_DUMP_DEBUG:
 	case FENCED_CMD_NODE_INFO:
 	case FENCED_CMD_DOMAIN_INFO:
-	case FENCED_CMD_DOMAIN_MEMBERS:
+	case FENCED_CMD_DOMAIN_NODES:
 		log_error("process_connection query on wrong socket");
 		break;
 	default:
@@ -556,8 +573,8 @@ static void *process_queries(void *arg)
 		case FENCED_CMD_DOMAIN_INFO:
 			query_domain_info(f);
 			break;
-		case FENCED_CMD_DOMAIN_MEMBERS:
-			query_domain_members(f, h.data);
+		case FENCED_CMD_DOMAIN_NODES:
+			query_domain_nodes(f, h.option, h.data);
 			break;
 		default:
 			break;
@@ -850,16 +867,16 @@ int main(int argc, char **argv)
 
 	read_arguments(argc, argv);
 
+	lockfile();
+
 	if (!daemon_debug_opt) {
 		if (daemon(0, 0) < 0) {
 			perror("main: cannot fork");
 			exit(EXIT_FAILURE);
 		}
-		chdir("/");
 		umask(0);
-		openlog("fenced", LOG_PID, LOG_DAEMON);
 	}
-	lockfile();
+	openlog("fenced", LOG_PID, LOG_DAEMON);
 	signal(SIGTERM, sigterm_handler);
 
 	set_oom_adj(-16);
@@ -876,7 +893,7 @@ void daemon_dump_save(void)
 	for (i = 0; i < len; i++) {
 		dump_buf[dump_point++] = daemon_debug_buf[i];
 
-		if (dump_point == DUMP_SIZE) {
+		if (dump_point == FENCED_DUMP_SIZE) {
 			dump_point = 0;
 			dump_wrap = 1;
 		}
@@ -890,7 +907,7 @@ int cman_quorate;
 int our_nodeid;
 char our_name[MAX_NODENAME_LEN+1];
 char daemon_debug_buf[256];
-char dump_buf[DUMP_SIZE];
+char dump_buf[FENCED_DUMP_SIZE];
 int dump_point;
 int dump_wrap;
 int group_mode;
