@@ -1,3 +1,16 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
+**  Copyright (C) 2004-2006 Red Hat, Inc.  All rights reserved.
+**
+**  This copyrighted material is made available to anyone wishing to use,
+**  modify, copy, or redistribute it subject to the terms and conditions
+**  of the GNU General Public License v.2.
+**
+*******************************************************************************
+******************************************************************************/
+
 #include <linux/kthread.h>
 #include <linux/sched.h>
 #include <linux/slab.h>
@@ -57,6 +70,9 @@ gfs_write_inode(struct inode *inode, int sync)
  * data.  So, sync it out.
  */
 
+// XXX FIX ME!
+
+#if 0
 static void
 gfs_put_inode(struct inode *inode)
 {
@@ -71,6 +87,7 @@ gfs_put_inode(struct inode *inode)
 	    !sdp->sd_args.ar_localcaching)
 		gfs_sync_page_i(inode, DIO_START | DIO_WAIT);
 }
+#endif
 
 /**
  * gfs_put_super - Unmount the filesystem
@@ -442,7 +459,7 @@ gfs_show_options(struct seq_file *s, struct vfsmount *mnt)
 
 struct super_operations gfs_super_ops = {
 	.write_inode = gfs_write_inode,
-	.put_inode = gfs_put_inode,
+	// XXX FIX ME .put_inode = gfs_put_inode,
 	.put_super = gfs_put_super,
 	.write_super = gfs_write_super,
 	.write_super_lockfs = gfs_write_super_lockfs,
