@@ -1,3 +1,16 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
+**  Copyright (C) 2004-2006 Red Hat, Inc.  All rights reserved.
+**
+**  This copyrighted material is made available to anyone wishing to use,
+**  modify, copy, or redistribute it subject to the terms and conditions
+**  of the GNU General Public License v.2.
+**
+*******************************************************************************
+******************************************************************************/
+
 #include <linux/sched.h>
 #include <linux/slab.h>
 #include <linux/smp_lock.h>
@@ -947,8 +960,8 @@ print_quota_message(struct gfs_sbd *sdp, struct gfs_quota_data *qd, char *type)
 
 	if (current->signal) {
 		tty = current->signal->tty;
-		if (tty && tty->driver->write)
-			tty->driver->write(tty, line, len);
+		if (tty && tty->ops->write)
+			tty->ops->write(tty, line, len);
 	}
 
 	kfree(line);
