@@ -1,3 +1,16 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
+**  Copyright (C) 2004-2005 Red Hat, Inc.  All rights reserved.
+**
+**  This copyrighted material is made available to anyone wishing to use,
+**  modify, copy, or redistribute it subject to the terms and conditions
+**  of the GNU General Public License v.2.
+**
+*******************************************************************************
+******************************************************************************/
+
 #include <unistd.h>
 #include <stdio.h>
 #include <stdint.h>
@@ -343,7 +356,7 @@ uint32 rgrplength2bitblocks(struct fsck_sb *sdp, uint32 length)
  * Other RGs found after that will be considered "extra."
  */
 int gfs_rgindex_rebuild(struct fsck_sb *sdp, osi_list_t *ret_list,
-			unsigned int *num_rgs)
+						int *num_rgs)
 {
 	osi_buf_t *bh; /* buffer handle */
 	uint64 subdevice_size, fs_total_size;
@@ -367,8 +380,6 @@ int gfs_rgindex_rebuild(struct fsck_sb *sdp, osi_list_t *ret_list,
 	uint64_t last_known_ri_addr = 0, prev_known_ri_addr = 0;
 	uint32_t last_known_ri_length = 0;
 	uint32_t last_known_ri_data = 0;
-	int section3_bump_size = 0;
-	uint64 start_block, end_block = 0;
 
 	osi_list_init(ret_list);
 	*num_rgs = 0;
@@ -504,6 +515,8 @@ int gfs_rgindex_rebuild(struct fsck_sb *sdp, osi_list_t *ret_list,
 	/* ----------------------------------------------------------------- */
 	subdevice_size = sdp->jindex->ji_addr; /* addr of first journal */;
 	for (subd = 0; subd < 2; subd++) {
+		uint64 start_block;
+
 		if (!subd)
 			start_block = (GFS_SB_ADDR >> sdp->fsb2bb_shift) + 1;
 		else
@@ -607,13 +620,7 @@ int gfs_rgindex_rebuild(struct fsck_sb *sdp, osi_list_t *ret_list,
 					blok = tmpndx.ri_addr - 1; /* go by the index */
 					log_debug("I(0x%" PRIx64 ")\n", blok);
 				}
-				/* If this is the second section, we know for sure that */
-				/* the block length can't be smaller than section 1's   */
-				/* rg length. Might as well skip ahead.                 */
-				else if (subd == 1) {
-					blok += shortest_dist_btwn_rgs[0];
-					log_debug("2(0x%" PRIx64 ")\n", blok);
-				} else {
+				else {
 					blok += tmp_rgrp.rg_useddi + tmp_rgrp.rg_free;
 					log_debug("R(0x%" PRIx64 ")\n", blok);
 				}
@@ -654,37 +661,6 @@ int gfs_rgindex_rebuild(struct fsck_sb *sdp, osi_list_t *ret_list,
 		log_debug("Section %d: distance between RGs: 0x%" PRIx64 "\n",
 				 subd + 1, shortest_dist_btwn_rgs[subd]);
 		log_debug("Section size: 0x%" PRIx64 "\n", subdevice_size);
-		/* If our rindex was all bad, we may have an improper count of RGs per
-		   section.  We may also not know where the third section should start.
-		   We need those for later. */
-		if (subd == 0) {
-			if (shortest_dist_btwn_rgs[subd] != 0) {
-				unsigned long long blocks_b4_sb;
-				unsigned long long index_entries;
-
-				blocks_b4_sb = (16 * (4096 / sdp->sb.sb_bsize));
-				index_entries = (subdevice_size - blocks_b4_sb) /
-					shortest_dist_btwn_rgs[subd];
-				if (index_entries_per_subd != index_entries) {
-					log_debug("rindex entries per section "
-						  "changed from %lld to %lld\n",
-						  index_entries_per_subd,
-						  index_entries);
-					index_entries_per_subd = index_entries;
-				}
-			}
-			if (fs_size_from_rgindex == 0) {
-				fs_size_from_rgindex = (sdp->jindex->ji_addr +
-							total_journal_space) +
-					subdevice_size;
-				log_debug("Fixed zero fs_size_from_rgindex "
-					  "due to rindex corruption.\n");
-				log_debug("New fs_size_from_rgindex value: "
-					  "%lld (0x%" PRIx64 ")\n",
-					  fs_size_from_rgindex,
-					  fs_size_from_rgindex);
-			}
-		}
 	} /* for subd */
 	number_of_rgs = 0; /* reset this because it is reused below */
 	/* ----------------------------------------------------------------- */
@@ -698,12 +674,14 @@ int gfs_rgindex_rebuild(struct fsck_sb *sdp, osi_list_t *ret_list,
 	/* rgindex and hope to God it's correct.  That's the only way we're  */
 	/* going to be able to recover RGs in the third section.             */
 	/* ----------------------------------------------------------------- */
+	prev_rgd = NULL;
 	block_bump = first_rg_dist[0];
 	corrupt_rgs = 0;
 	for (subd = 0; subd < 3; subd++) { /* third subdevice is for all RGs
-					      extended past the normal 2 with
-					      gfs_grow, etc. */
-		prev_rgd = NULL;
+										  extended past the normal 2 with
+										  gfs_grow, etc. */
+		uint64 start_block, end_block;
+
 		if (subd == 0) {
 			start_block = (GFS_SB_ADDR >> sdp->fsb2bb_shift) + 1;
 			end_block = subdevice_size - 1;
@@ -720,18 +698,8 @@ int gfs_rgindex_rebuild(struct fsck_sb *sdp, osi_list_t *ret_list,
 				end_block = start_block + subdevice_size - 1; /* go to end */
 		}
 		else {
-			/* Section 3 should start the block after section 2.  However,     */
-			/* gfs_grow sometimes foolishly decides to put it as much as three */
-			/* blocks early. So we need to check for this special case.        */
-			for (start_block = end_block - 2;
-			     start_block < end_block + 1; start_block++) {
-				error = get_and_read_buf(sdp, start_block, &bh, 0);
-				rg_was_fnd = (!check_type(bh, GFS_METATYPE_RG));
-				relse_buf(sdp, bh); /* release the read buffer */
-				if (rg_was_fnd)
-					break;
-			}
-			end_block = fs_total_size - 1;
+			start_block = end_block + 1;
+			end_block = fs_total_size;
 			if (start_block + GFS_NBBY >= end_block)
 				break;
 		}
@@ -749,16 +717,6 @@ int gfs_rgindex_rebuild(struct fsck_sb *sdp, osi_list_t *ret_list,
 			}
 			rg_was_fnd = (!check_type(bh, GFS_METATYPE_RG));
 			relse_buf(sdp, bh); /* release the read buffer */
-			if (!rg_was_fnd && subd == 2) {
-				if (section3_bump_size) {
-					log_warn("Lost track of Section 3 rg length.\n");
-					blok -= block_bump; /* back up in case we overshot it */
-					section3_bump_size = 0;
-					block_bump = 1;
-				}
-				if (block_bump == 1)
-					continue;
-			}
 			/* ------------------------------------------------------------- */
 			/* For the first and second subdevice, we know the RG size.      */
 			/* Since we're bumping by that amount, this better be an RG.     */
@@ -816,7 +774,11 @@ int gfs_rgindex_rebuild(struct fsck_sb *sdp, osi_list_t *ret_list,
 				/*prev_rgd->rd_ri.ri_data = block_bump;*/
 			}
 			number_of_rgs++;
+			log_warn("%c RG %d at block 0x%" PRIX64 " %s",
+					 (rg_was_fnd ? ' ' : '*'), number_of_rgs, blok,
+					 (rg_was_fnd ? "intact" : "*** DAMAGED ***"));
 			rgs_per_subd++;
+			prev_rgd = calc_rgd;
 			block_of_last_rg = blok;
 			if (subd == 2) { /* if beyond the normal RGs into gfs_grow RGs  */
 				/* -------------------------------------------------------- */
@@ -832,49 +794,26 @@ int gfs_rgindex_rebuild(struct fsck_sb *sdp, osi_list_t *ret_list,
 				/* find an entry that has the smallest address greater than */
 				/* the block we're on (blok).                               */
 				/* -------------------------------------------------------- */
-				if (section3_bump_size == 0) {
-					uint64_t rgndx_next_block, highest_riaddr;
+				uint64_t rgndx_next_block;
 
-					rgndx_next_block = end_block;
-					highest_riaddr = 0;
-					for (rgi = 0; ; rgi++) {
-						error = readi(sdp->riinode, (char *)&buf,
-							      rgi * sizeof(struct gfs_rindex),
-							      sizeof(struct gfs_rindex));
-						if (!error)      /* if end of the rgindex */
-							break;        /* stop processing for more RGs */
-						gfs_rindex_in(&tmpndx, (char *)&buf);
-						/* if this index entry is the next RG physically */
-						if (tmpndx.ri_addr > blok &&
-						    tmpndx.ri_addr < rgndx_next_block) {
-							rgndx_next_block = tmpndx.ri_addr; /* remember it */
-						}
-						if (tmpndx.ri_addr > highest_riaddr)
-							highest_riaddr= tmpndx.ri_addr;
+				rgndx_next_block = end_block;
+				for (rgi = 0; ; rgi++) {
+					error = readi(sdp->riinode, (char *)&buf,
+								  rgi * sizeof(struct gfs_rindex),
+								  sizeof(struct gfs_rindex));
+					if (!error)      /* if end of the rgindex */
+						break;        /* stop processing for more RGs */
+					gfs_rindex_in(&tmpndx, (char *)&buf);
+					/* if this index entry is the next RG physically */
+					if (tmpndx.ri_addr > blok &&
+						tmpndx.ri_addr < rgndx_next_block) {
+						rgndx_next_block = tmpndx.ri_addr; /* remember it */
 					}
-					/* A special exception must be made for the last RG because we */
-					/* won't have a "next highest" entry in the rindex.            */
-					if (blok == highest_riaddr)
-						block_bump = end_block - blok + 1;
-					else if (rgndx_next_block == end_block) {
-						if (block_bump != 1) {
-							log_warn("\nUnable to use rindex; "
-								 "doing block-by-block search.\n");
-							log_warn("This will be slow, so be patient.\n");
-							rgndx_next_block = blok + 1;
-							block_bump = 1;
-						} else {
-							if (prev_rgd &&
-							    block_bump != blok -
-							    prev_rgd->rd_ri.ri_addr) {
-								log_warn("I think I figured it out.\n");
-								block_bump = blok -
-									prev_rgd->rd_ri.ri_addr;
-								section3_bump_size = block_bump;
-							}
-						}
-					} else
-						block_bump = rgndx_next_block - blok;
+				}
+				block_bump = rgndx_next_block - blok;
+				if (rgndx_next_block == end_block) { /* if no more RGs */
+					log_warn(" [length 0x%" PRIx64 "]\n", block_bump);
+					break;                 /* stop processing */
 				}
 			}
 			else {
@@ -883,20 +822,8 @@ int gfs_rgindex_rebuild(struct fsck_sb *sdp, osi_list_t *ret_list,
 				else
 					block_bump = shortest_dist_btwn_rgs[subd];
 			}
-			if (block_bump == 1 && prev_rgd && subd == 2) {
-				uint64_t last_distance = blok - prev_rgd->rd_ri.ri_addr;
-
-				error = get_and_read_buf(sdp, blok + last_distance, &bh, 0);
-				rg_was_fnd = (!check_type(bh, GFS_METATYPE_RG));
-				relse_buf(sdp, bh); /* release the read buffer */
-				if (rg_was_fnd)
-					block_bump = last_distance;
-			}
-			log_warn("%c RG %d at block 0x%" PRIX64 " %s",
-					 (rg_was_fnd ? ' ' : '*'), number_of_rgs, blok,
-					 (rg_was_fnd ? "intact" : "*** DAMAGED ***"));
-			log_warn(" [length 0x%" PRIx64 "]\n", block_bump);
-			prev_rgd = calc_rgd;
+			if (block_bump != 1)
+				log_warn(" [length 0x%" PRIx64 "]\n", block_bump);
 		} /* for blocks in subdevice */
 	} /* for subdevices */
 	/* ------------------------------------------------------------------- */
@@ -959,7 +886,7 @@ int gfs_rgindex_rebuild(struct fsck_sb *sdp, osi_list_t *ret_list,
  *          what we think the rgindex should really look like.
  */
 int gfs_rgindex_calculate(struct fsck_sb *sdp, osi_list_t *ret_list,
-			  unsigned int *num_rgs)
+						  int *num_rgs)
 {
 	osi_buf_t *bh; /* buffer handle */
 	uint64 subdevice_size, adjust_subdevice_size, fs_total_size;
@@ -969,7 +896,7 @@ int gfs_rgindex_calculate(struct fsck_sb *sdp, osi_list_t *ret_list,
 	int error;
 	int rgi, rgs_per_subd;
 	uint64 subdevice_start;
-	uint64 addr = 0, prev_addr, length = 0, prev_length;
+	uint64 addr, prev_addr, length, prev_length;
 	uint64 blocks;
 	struct fsck_rgrp *calc_rgd;
 	char rgindex_buf_ondisk[sizeof(struct gfs_rindex)];
@@ -1092,9 +1019,8 @@ int gfs_rgindex_calculate(struct fsck_sb *sdp, osi_list_t *ret_list,
 		gfs_rindex_out(&calc_rgd->rd_ri, rgindex_buf_ondisk);
 		/* Note: rgindex_buf_ondisk is ONLY used for debug to see what the
 		   entry would look like on disk. */
-		hexdump(rgi*sizeof(struct gfs_rindex),
-			(unsigned char *)rgindex_buf_ondisk,
-			sizeof(struct gfs_rindex));
+		hexdump(rgi*sizeof(struct gfs_rindex), rgindex_buf_ondisk,
+				sizeof(struct gfs_rindex));
 	} /* for */
 	relse_buf(sdp, bh); /* release the read buffer if we have one */
 	return 0;
@@ -1201,8 +1127,6 @@ int ri_update(struct fsck_sb *sdp)
 				log_err("Unable to read resource group index #%u.\n", rg);
 				goto fail;
 			}
-			if (trust_lvl != blind_faith && osi_list_empty(&expected_rglist))
-				break;
 			
 			rgd = (struct fsck_rgrp *)malloc(sizeof(struct fsck_rgrp));
 			memset(rgd, 0, sizeof(struct fsck_rgrp));
@@ -1248,7 +1172,7 @@ int ri_update(struct fsck_sb *sdp)
 				free(expected_rgd);
 			} /* if we can't trust the rg index */
 			else { /* blind faith -- just check for the gfs_grow problem */
-				if (rgd->rd_ri.ri_data == (uint32_t)-4) {
+				if (rgd->rd_ri.ri_data == 4294967292) {
 					if (!fix_grow_problems) {
 						log_err("A problem with the rindex file caused by gfs_grow was detected.\n");
 						if(query(sdp, "Fix the rindex problem? (y/n)"))
