@@ -1,3 +1,15 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
+**  Copyright (C) 2004-2007 Red Hat, Inc.  All rights reserved.
+**
+**  This copyrighted material is made available to anyone wishing to use,
+**  modify, copy, or redistribute it subject to the terms and conditions
+**  of the GNU General Public License v.2.
+**
+*******************************************************************************
+******************************************************************************/
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -244,33 +256,25 @@ print_quota(commandline_t *comline,
 	case GQ_UNITS_KILOBYTE:
 		if (sb->sb_bsize == 512)
 			printf("limit: %-10llu warn: %-10lluvalue: %-10llu\n",
-			       (unsigned long long)q->qu_limit / 2,
-			       (unsigned long long)q->qu_warn / 2,
-			       (unsigned long long)q->qu_value / 2);
+			       q->qu_limit / 2,
+			       q->qu_warn / 2,
+			       q->qu_value / 2);
 		else
 			printf("limit: %-10llu warn: %-10lluvalue: %-10llu\n",
-			       (unsigned long long)
 			       q->qu_limit << (sb->sb_bsize_shift - 10),
-			       (unsigned long long)
 			       q->qu_warn << (sb->sb_bsize_shift - 10),
-			       (unsigned long long)
 			       q->qu_value << (sb->sb_bsize_shift - 10));
 		break;
 
 	case GQ_UNITS_FSBLOCK:
 		printf("limit: %-10llu warn: %-10llu value: %-10llu\n",
-		       (unsigned long long)q->qu_limit,
-		       (unsigned long long)q->qu_warn,
-		       (unsigned long long)q->qu_value);
+		       q->qu_limit, q->qu_warn, q->qu_value);
 		break;
 
 	case GQ_UNITS_BASICBLOCK:
 		printf("limit: %-10llu warn: %-10llu value: %-10llu\n",
-		       (unsigned long long)
 		       q->qu_limit << (sb->sb_bsize_shift - 9),
-		       (unsigned long long)
 		       q->qu_warn << (sb->sb_bsize_shift - 9),
-		       (unsigned long long)
 		       q->qu_value << (sb->sb_bsize_shift - 9));
 		break;
 
@@ -316,8 +320,7 @@ read_quota_internal(int fd, uint32_t id, int id_type, struct gfs2_quota *q)
 		die("failed to read from quota file: %s\n", strerror(errno));
 	if (error != sizeof(struct gfs2_quota))
 		die("Couldn't read %lu bytes from quota file at offset %llu\n",
-		    (unsigned long)sizeof(struct gfs2_quota),
-		    (unsigned long long)offset);
+		    sizeof(struct gfs2_quota), (unsigned long long)offset);
 	gfs2_quota_in(q, buf);
 }
 
@@ -498,8 +501,10 @@ do_reset(struct gfs2_sbd *sdp, commandline_t *comline)
 	strcpy(sdp->path_name, comline->filesystem);
 	check_for_gfs2(sdp);
 	read_superblock(&sdp->sd_sb, sdp);
-	mount_gfs2_meta(sdp);
-
+	if (!find_gfs2_meta(sdp))
+		mount_gfs2_meta(sdp);
+	lock_for_admin(sdp);
+	
 	strcpy(quota_file, sdp->metafs_path);
 	strcat(quota_file, "/quota");
 
@@ -555,8 +560,10 @@ do_list(struct gfs2_sbd *sdp, commandline_t *comline)
 	strcpy(sdp->path_name, comline->filesystem);
 	check_for_gfs2(sdp);
 	read_superblock(&sdp->sd_sb, sdp);
-	mount_gfs2_meta(sdp);
-
+	if (!find_gfs2_meta(sdp))
+		mount_gfs2_meta(sdp);
+	lock_for_admin(sdp);
+	
 	strcpy(quota_file, sdp->metafs_path);
 	strcat(quota_file, "/quota");
 
@@ -642,8 +649,10 @@ do_get_one(struct gfs2_sbd *sdp, commandline_t *comline, char *filesystem)
 	strcpy(sdp->path_name, filesystem);
 	check_for_gfs2(sdp);
 	read_superblock(&sdp->sd_sb, sdp);
-	mount_gfs2_meta(sdp);
-
+	if (!find_gfs2_meta(sdp))
+		mount_gfs2_meta(sdp);
+	lock_for_admin(sdp);
+	
 	strcpy(quota_file, sdp->metafs_path);
 	strcat(quota_file, "/quota");
 
@@ -803,8 +812,10 @@ do_set(struct gfs2_sbd *sdp, commandline_t *comline)
 	strcpy(sdp->path_name, comline->filesystem);
 	check_for_gfs2(sdp);
 	read_superblock(&sdp->sd_sb, sdp);
-	mount_gfs2_meta(sdp);
-
+	if (!find_gfs2_meta(sdp))
+		mount_gfs2_meta(sdp);
+	lock_for_admin(sdp);
+	
 	strcpy(quota_file, sdp->metafs_path);
 	strcat(quota_file, "/quota");
 
@@ -961,6 +972,7 @@ main(int argc, char *argv[])
 	commandline_t comline;
 
 	prog_name = argv[0];
+	sdp->metafs_mounted = 0;
 
 	memset(sdp, 0, sizeof(struct gfs2_sbd));
 	memset(&comline, 0, sizeof(commandline_t));
