@@ -36,8 +36,11 @@ struct node {
 	int check_fencing;
 	int check_quorum;
 	int check_fs;
-	int fs_notify;
+	int fs_notified;
 	uint64_t add_time;
+	uint32_t added_seq;	/* for queries */
+	uint32_t removed_seq;	/* for queries */
+	int failed_reason;	/* for queries */
 };
 
 /* One of these change structs is created for every confchg a cpg gets. */
@@ -55,7 +58,8 @@ struct change {
 	int failed_count;
 	int state;
 	int we_joined;
-	uint32_t seq; /* just used as a reference when debugging */
+	uint32_t seq; /* used as a reference for debugging, and for queries */
+	uint32_t combined_seq; /* for queries */
 };
 
 char *msg_name(int type)
@@ -289,13 +293,14 @@ static struct node *get_node_history(struct lockspace *ls, int nodeid)
 	return NULL;
 }
 
-static void node_history_init(struct lockspace *ls, int nodeid)
+static void node_history_init(struct lockspace *ls, int nodeid,
+			      struct change *cg)
 {
 	struct node *node;
 
 	node = get_node_history(ls, nodeid);
 	if (node)
-		return;
+		goto out;
 
 	node = malloc(sizeof(struct node));
 	if (!node)
@@ -305,6 +310,8 @@ static void node_history_init(struct lockspace *ls, int nodeid)
 	node->nodeid = nodeid;
 	node->add_time = 0;
 	list_add_tail(&node->list, &ls->node_history);
+ out:
+	node->added_seq = cg->seq;	/* for queries */
 }
 
 static void node_history_start(struct lockspace *ls, int nodeid)
@@ -320,7 +327,8 @@ static void node_history_start(struct lockspace *ls, int nodeid)
 	node->add_time = time(NULL);
 }
 
-static void node_history_left(struct lockspace *ls, int nodeid)
+static void node_history_left(struct lockspace *ls, int nodeid,
+			      struct change *cg)
 {
 	struct node *node;
 
@@ -331,9 +339,11 @@ static void node_history_left(struct lockspace *ls, int nodeid)
 	}
 
 	node->add_time = 0;
+	node->removed_seq = cg->seq;	/* for queries */
 }
 
-static void node_history_fail(struct lockspace *ls, int nodeid)
+static void node_history_fail(struct lockspace *ls, int nodeid,
+			      struct change *cg, int reason)
 {
 	struct node *node;
 
@@ -343,11 +353,16 @@ static void node_history_fail(struct lockspace *ls, int nodeid)
 		return;
 	}
 
-	if (!node->add_time)
+	if (cfgd_enable_fencing && !node->add_time)
 		node->check_fencing = 1;
 
-	node->check_quorum = 1;
+	if (cfgd_enable_quorum)
+		node->check_quorum = 1;
+
 	node->check_fs = 1;
+
+	node->removed_seq = cg->seq;	/* for queries */
+	node->failed_reason = reason;	/* for queries */
 }
 
 static int check_fencing_done(struct lockspace *ls)
@@ -358,6 +373,9 @@ static int check_fencing_done(struct lockspace *ls)
 	int wait_count = 0;
 	int rv;
 
+	if (!cfgd_enable_fencing)
+		return 1;
+
 	list_for_each_entry(node, &ls->node_history, list) {
 		if (!node->check_fencing)
 			continue;
@@ -399,11 +417,16 @@ static int check_fencing_done(struct lockspace *ls)
 	return 1;
 }
 
+/* wait for cman to see all the same nodes failed, and to say there's quorum */
+
 static int check_quorum_done(struct lockspace *ls)
 {
 	struct node *node;
 	int wait_count = 0;
 
+	if (!cfgd_enable_quorum)
+		return 1;
+
 	if (!cman_quorate) {
 		log_group(ls, "check_quorum %d", cman_quorate);
 		return 0;
@@ -429,6 +452,8 @@ static int check_quorum_done(struct lockspace *ls)
 	return 1;
 }
 
+/* wait for local fs_controld to ack each failed node */
+
 static int check_fs_done(struct lockspace *ls)
 {
 	struct node *node;
@@ -442,7 +467,7 @@ static int check_fs_done(struct lockspace *ls)
 		if (!node->check_fs)
 			continue;
 
-		if (node->fs_notify) {
+		if (node->fs_notified) {
 			node->check_fs = 0;
 		} else {
 			log_group(ls, "check_fs %d needs fs notify",
@@ -604,7 +629,14 @@ static void cleanup_changes(struct lockspace *ls)
 		free_cg(ls->started_change);
 	ls->started_change = cg;
 
+	ls->started_count++;
+	if (!ls->started_count)
+		ls->started_count++;
+
+	cg->combined_seq = cg->seq; /* for queries */
+
 	list_for_each_entry_safe(cg, safe, &ls->changes, list) {
+		ls->started_change->combined_seq = cg->seq; /* for queries */
 		list_del(&cg->list);
 		free_cg(cg);
 	}
@@ -658,24 +690,30 @@ static void set_plock_ckpt_node(struct lockspace *ls)
 	ls->plock_ckpt_node = low;
 }
 
+#define COUNT_NUMS 5
+
 /* do the change details in the message match the details of the given change */
 
 static int match_change(struct lockspace *ls, struct change *cg,
-			struct dlm_header *hd, int len)
+			struct dlm_header *hd, int len, uint32_t *started)
 {
 	struct member *memb;
 	int member_count, joined_count, remove_count, failed_count;
 	int i, n_ints, *nums, nodeid, members_mismatch;
 	uint32_t seq = hd->msgdata;
+	uint32_t started_count;
 
 	nums = (int *)((char *)hd + sizeof(struct dlm_header));
 
-	member_count = le32_to_cpu(nums[0]);
-	joined_count = le32_to_cpu(nums[1]);
-	remove_count = le32_to_cpu(nums[2]);
-	failed_count = le32_to_cpu(nums[3]);
+	started_count = le32_to_cpu(nums[0]);
+	member_count = le32_to_cpu(nums[1]);
+	joined_count = le32_to_cpu(nums[2]);
+	remove_count = le32_to_cpu(nums[3]);
+	failed_count = le32_to_cpu(nums[4]);
 
-	n_ints = 4 + member_count;
+	*started = started_count;
+
+	n_ints = COUNT_NUMS + member_count;
 	if (len < (sizeof(struct dlm_header) + (n_ints * sizeof(int)))) {
 		log_group(ls, "match_change fail %d:%u bad len %d nums %s",
 			  hd->nodeid, seq, len, str_nums(nums, n_ints));
@@ -687,7 +725,7 @@ static int match_change(struct lockspace *ls, struct change *cg,
 	   get messages for changes prior to the change in which we're added. */
 
 	for (i = 0; i < member_count; i++) {
-		if (our_nodeid == le32_to_cpu(nums[4+i]))
+		if (our_nodeid == le32_to_cpu(nums[COUNT_NUMS + i]))
 			break;
 	}
 	if (i == member_count) {
@@ -721,7 +759,7 @@ static int match_change(struct lockspace *ls, struct change *cg,
 
 	members_mismatch = 0;
 	for (i = 0; i < member_count; i++) {
-		nodeid = le32_to_cpu(nums[4+i]);
+		nodeid = le32_to_cpu(nums[COUNT_NUMS + i]);
 		memb = find_memb(cg, nodeid);
 		if (memb)
 			continue;
@@ -732,6 +770,7 @@ static int match_change(struct lockspace *ls, struct change *cg,
 	if (members_mismatch)
 		return 0;
 
+	log_group(ls, "match_change done %d:%u", hd->nodeid, seq);
 	return 1;
 }
 
@@ -743,7 +782,7 @@ static void send_plocks_stored(struct lockspace *ls)
 	int n_ints, len, *p, i;
 	char *buf;
 
-	n_ints = 4 + cg->member_count;
+	n_ints = COUNT_NUMS + cg->member_count;
 	len = sizeof(struct dlm_header) + (n_ints * sizeof(uint32_t));
 
 	buf = malloc(len);
@@ -762,12 +801,13 @@ static void send_plocks_stored(struct lockspace *ls)
 	   us more certainty in matching stopped messages to the correct
 	   change that they are for */
 
-	p[0] = cpu_to_le32(cg->member_count);
-	p[1] = cpu_to_le32(cg->joined_count);
-	p[2] = cpu_to_le32(cg->remove_count);
-	p[3] = cpu_to_le32(cg->failed_count);
+	p[0] = cpu_to_le32(ls->started_count);
+	p[1] = cpu_to_le32(cg->member_count);
+	p[2] = cpu_to_le32(cg->joined_count);
+	p[3] = cpu_to_le32(cg->remove_count);
+	p[4] = cpu_to_le32(cg->failed_count);
 
-	i = 4;
+	i = COUNT_NUMS;
 	list_for_each_entry(memb, &cg->members, list)
 		p[i++] = cpu_to_le32(memb->nodeid);
 
@@ -779,6 +819,8 @@ static void send_plocks_stored(struct lockspace *ls)
 static void receive_plocks_stored(struct lockspace *ls, struct dlm_header *hd,
 				  int len)
 {
+	uint32_t started_count;
+
 	log_group(ls, "receive_plocks_stored %d:%u need_plocks %d",
 		  hd->nodeid, hd->msgdata, ls->need_plocks);
 
@@ -790,7 +832,7 @@ static void receive_plocks_stored(struct lockspace *ls, struct dlm_header *hd,
 	   the next plocks_stored msg following the current start */
 	   
 	if (!list_empty(&ls->changes) || !ls->started_change ||
-	    !match_change(ls, ls->started_change, hd, len)) {
+	    !match_change(ls, ls->started_change, hd, len, &started_count)) {
 		log_group(ls, "receive_plocks_stored %d:%u ignore",
 			  hd->nodeid, hd->msgdata);
 		return;
@@ -822,12 +864,12 @@ static void receive_plocks_stored(struct lockspace *ls, struct dlm_header *hd,
    for confchg1 or confchg2?  Hopefully by comparing the counts and members. */
 
 static struct change *find_change(struct lockspace *ls, struct dlm_header *hd,
-				  int len)
+				  int len, uint32_t *started_count)
 {
 	struct change *cg;
 
 	list_for_each_entry_reverse(cg, &ls->changes, list) {
-		if (!match_change(ls, cg, hd, len))
+		if (!match_change(ls, cg, hd, len, started_count))
 			continue;
 		return cg;
 	}
@@ -836,27 +878,31 @@ static struct change *find_change(struct lockspace *ls, struct dlm_header *hd,
 	return NULL;
 }
 
-/* We require new members (memb->added) to be joining the lockspace
-   (memb->joining).  New members that are not joining the lockspace can happen
-   when the cpg partitions and is then merged back together (shouldn't happen
-   in general, but is possible).  We label these new members that are not
-   joining as "disallowed", and ignore their start message. */
+static int is_added(struct lockspace *ls, int nodeid)
+{
+	struct change *cg;
+	struct member *memb;
 
-/* Handle spurious joins by ignoring this start message if the node says it's
-   not joining (i.e. it's already a member), but we see it being added (i.e.
-   it's not already a member) */
+	list_for_each_entry(cg, &ls->changes, list) {
+		memb = find_memb(cg, nodeid);
+		if (memb && memb->added)
+			return 1;
+	}
+	return 0;
+}
 
 static void receive_start(struct lockspace *ls, struct dlm_header *hd, int len)
 {
 	struct change *cg;
 	struct member *memb;
-	int joining = 0;
 	uint32_t seq = hd->msgdata;
+	uint32_t started_count;
+	int added;
 
 	log_group(ls, "receive_start %d:%u flags %x len %d", hd->nodeid, seq,
 		  hd->flags, len);
 
-	cg = find_change(ls, hd, len);
+	cg = find_change(ls, hd, len, &started_count);
 	if (!cg)
 		return;
 
@@ -869,17 +915,23 @@ static void receive_start(struct lockspace *ls, struct dlm_header *hd, int len)
 
 	memb->start_flags = hd->flags;
 
-	if (memb->start_flags & DLM_MFLG_JOINING)
-		joining = 1;
+	added = is_added(ls, hd->nodeid);
 
-	if ((memb->added && !joining) || (!memb->added && joining)) {
-		log_error("receive_start %d:%u disallowed added %d joining %d",
-			  hd->nodeid, seq, memb->added, joining);
+	if (added && started_count) {
+		log_error("receive_start %d:%u add node with started_count %u",
+			  hd->nodeid, seq, started_count);
+
+		/* observe this scheme working before using it; I'm not sure
+		   that a joining node won't ever see an existing node as added
+		   under normal circumstances */
+		/*
 		memb->disallowed = 1;
-	} else {
-		node_history_start(ls, hd->nodeid);
-		memb->start = 1;
+		return;
+		*/
 	}
+
+	node_history_start(ls, hd->nodeid);
+	memb->start = 1;
 }
 
 static void send_start(struct lockspace *ls)
@@ -890,7 +942,7 @@ static void send_start(struct lockspace *ls)
 	int n_ints, len, *p, i;
 	char *buf;
 
-	n_ints = 4 + cg->member_count;
+	n_ints = COUNT_NUMS + cg->member_count;
 	len = sizeof(struct dlm_header) + (n_ints * sizeof(int));
 
 	buf = malloc(len);
@@ -903,7 +955,7 @@ static void send_start(struct lockspace *ls)
 	hd->type = DLM_MSG_START;
 	hd->msgdata = cg->seq;
 
-	if (cg->we_joined)
+	if (ls->joining)
 		hd->flags |= DLM_MFLG_JOINING;
 
 	if (!ls->need_plocks)
@@ -915,18 +967,19 @@ static void send_start(struct lockspace *ls)
 	   us more certainty in matching stopped messages to the correct
 	   change that they are for */
 
-	p[0] = cpu_to_le32(cg->member_count);
-	p[1] = cpu_to_le32(cg->joined_count);
-	p[2] = cpu_to_le32(cg->remove_count);
-	p[3] = cpu_to_le32(cg->failed_count);
+	p[0] = cpu_to_le32(ls->started_count);
+	p[1] = cpu_to_le32(cg->member_count);
+	p[2] = cpu_to_le32(cg->joined_count);
+	p[3] = cpu_to_le32(cg->remove_count);
+	p[4] = cpu_to_le32(cg->failed_count);
 
-	i = 4;
+	i = COUNT_NUMS;
 	list_for_each_entry(memb, &cg->members, list)
 		p[i++] = cpu_to_le32(memb->nodeid);
 
-	log_group(ls, "send_start %u flags %x counts %d %d %d %d", cg->seq,
-		  hd->flags, cg->member_count, cg->joined_count,
-		  cg->remove_count, cg->failed_count);
+	log_group(ls, "send_start %u flags %x counts %u %d %d %d %d",
+		  cg->seq, hd->flags, ls->started_count, cg->member_count,
+		  cg->joined_count, cg->remove_count, cg->failed_count);
 
 	dlm_send_message(ls, buf, len);
 
@@ -1059,8 +1112,10 @@ static int add_change(struct lockspace *ls,
 	memset(cg, 0, sizeof(struct change));
 	INIT_LIST_HEAD(&cg->members);
 	INIT_LIST_HEAD(&cg->removed);
-	cg->seq = ++ls->change_seq;
 	cg->state = CGST_WAIT_CONDITIONS;
+	cg->seq = ++ls->change_seq;
+	if (!cg->seq)
+		cg->seq = ++ls->change_seq;
 
 	cg->member_count = member_list_entries;
 	cg->joined_count = joined_list_entries;
@@ -1089,9 +1144,10 @@ static int add_change(struct lockspace *ls,
 		list_add_tail(&memb->list, &cg->removed);
 
 		if (memb->failed)
-			node_history_fail(ls, memb->nodeid);
+			node_history_fail(ls, memb->nodeid, cg,
+					  left_list[i].reason);
 		else
-			node_history_left(ls, memb->nodeid);
+			node_history_left(ls, memb->nodeid, cg);
 
 		log_group(ls, "add_change %u nodeid %d remove reason %d",
 			  cg->seq, memb->nodeid, left_list[i].reason);
@@ -1109,15 +1165,17 @@ static int add_change(struct lockspace *ls,
 		if (memb->nodeid == our_nodeid)
 			cg->we_joined = 1;
 		else
-			node_history_init(ls, memb->nodeid);
+			node_history_init(ls, memb->nodeid, cg);
 
 		log_group(ls, "add_change %u nodeid %d joined", cg->seq,
 			  memb->nodeid);
 	}
 
-	if (cg->we_joined)
+	if (cg->we_joined) {
+		log_group(ls, "add_change %u we joined", cg->seq);
 		list_for_each_entry(memb, &cg->members, list)
-			node_history_init(ls, memb->nodeid);
+			node_history_init(ls, memb->nodeid, cg);
+	}
 
 	log_group(ls, "add_change %u member %d joined %d remove %d failed %d",
 		  cg->seq, cg->member_count, cg->joined_count, cg->remove_count,
@@ -1439,3 +1497,215 @@ int setup_cpg(void)
 	return 0;
 }
 
+/* fs_controld has seen nodedown for nodeid; it's now ok for dlm to do
+   recovery for the failed node */
+
+int set_fs_notified(struct lockspace *ls, int nodeid)
+{
+	struct node *node;
+
+	/* this shouldn't happen */
+	node = get_node_history(ls, nodeid);
+	if (!node)
+		return -ESRCH;
+
+	/* this can happen, we haven't seen a nodedown for this node yet,
+	   but we should soon */
+	if (!node->check_fs)
+		return -EAGAIN;
+
+	node->fs_notified = 1;
+	return 0;
+}
+
+int set_lockspace_info(struct lockspace *ls, struct dlmc_lockspace *lockspace)
+{
+	struct change *cg, *last = NULL;
+
+	strncpy(lockspace->name, ls->name, DLM_LOCKSPACE_LEN);
+	lockspace->global_id = ls->global_id;
+
+	if (ls->joining)
+		lockspace->flags |= DLMC_LF_JOINING;
+	if (ls->leaving)
+		lockspace->flags |= DLMC_LF_LEAVING;
+	if (ls->kernel_stopped)
+		lockspace->flags |= DLMC_LF_KERNEL_STOPPED;
+	if (ls->fs_registered)
+		lockspace->flags |= DLMC_LF_FS_REGISTERED;
+	if (ls->need_plocks)
+		lockspace->flags |= DLMC_LF_NEED_PLOCKS;
+	if (ls->save_plocks)
+		lockspace->flags |= DLMC_LF_SAVE_PLOCKS;
+
+	if (!ls->started_change)
+		goto next;
+
+	cg = ls->started_change;
+
+	lockspace->cg_prev.member_count = cg->member_count;
+	lockspace->cg_prev.joined_count = cg->joined_count;
+	lockspace->cg_prev.remove_count = cg->remove_count;
+	lockspace->cg_prev.failed_count = cg->failed_count;
+	lockspace->cg_prev.combined_seq = cg->combined_seq;
+	lockspace->cg_prev.seq = cg->seq;
+
+ next:
+	if (list_empty(&ls->changes))
+		goto out;
+
+	list_for_each_entry(cg, &ls->changes, list)
+		last = cg;
+
+	cg = list_first_entry(&ls->changes, struct change, list);
+
+	lockspace->cg_next.member_count = cg->member_count;
+	lockspace->cg_next.joined_count = cg->joined_count;
+	lockspace->cg_next.remove_count = cg->remove_count;
+	lockspace->cg_next.failed_count = cg->failed_count;
+	lockspace->cg_next.combined_seq = last->seq;
+	lockspace->cg_next.seq = cg->seq;
+
+	if (cg->state == CGST_WAIT_CONDITIONS)
+		lockspace->cg_next.wait_condition = 4;
+	if (poll_fencing)
+		lockspace->cg_next.wait_condition = 1;
+	else if (poll_quorum)
+		lockspace->cg_next.wait_condition = 2;
+	else if (poll_fs)
+		lockspace->cg_next.wait_condition = 3;
+
+	if (cg->state == CGST_WAIT_MESSAGES)
+		lockspace->cg_next.wait_messages = 1;
+ out:
+	return 0;
+}
+
+static int _set_node_info(struct lockspace *ls, struct change *cg, int nodeid,
+			  struct dlmc_node *node)
+{
+	struct member *m = NULL;
+	struct node *n;
+
+	node->nodeid = nodeid;
+
+	if (cg)
+		m = find_memb(cg, nodeid);
+	if (!m)
+		goto history;
+
+	node->flags |= DLMC_NF_MEMBER;
+
+	if (m->start)
+		node->flags |= DLMC_NF_START;
+	if (m->disallowed)
+		node->flags |= DLMC_NF_DISALLOWED;
+
+ history:
+	n = get_node_history(ls, nodeid);
+	if (!n)
+		goto out;
+
+	if (n->check_fencing)
+		node->flags |= DLMC_NF_CHECK_FENCING;
+	if (n->check_quorum)
+		node->flags |= DLMC_NF_CHECK_QUORUM;
+	if (n->check_fs)
+		node->flags |= DLMC_NF_CHECK_FS;
+	if (n->fs_notified)
+		node->flags |= DLMC_NF_FS_NOTIFIED;
+
+	node->added_seq = n->added_seq;
+	node->removed_seq = n->removed_seq;
+	node->failed_reason = n->failed_reason;
+ out:
+	return 0;
+}
+
+int set_node_info(struct lockspace *ls, int nodeid, struct dlmc_node *node)
+{
+	struct change *cg;
+
+	if (!list_empty(&ls->changes)) {
+		cg = list_first_entry(&ls->changes, struct change, list);
+		return _set_node_info(ls, cg, nodeid, node);
+	}
+
+	return _set_node_info(ls, ls->started_change, nodeid, node);
+}
+
+int set_lockspaces(int *count, struct dlmc_lockspace **lss_out)
+{
+	struct lockspace *ls;
+	struct dlmc_lockspace *lss, *lsp;
+	int ls_count = 0;
+
+	list_for_each_entry(ls, &lockspaces, list)
+		ls_count++;
+
+	lss = malloc(ls_count * sizeof(struct dlmc_lockspace));
+	if (!lss)
+		return -ENOMEM;
+	memset(lss, 0, ls_count * sizeof(struct dlmc_lockspace));
+
+	lsp = lss;
+	list_for_each_entry(ls, &lockspaces, list) {
+		set_lockspace_info(ls, lsp++);
+	}
+
+	*count = ls_count;
+	*lss_out = lss;
+	return 0;
+}
+
+int set_lockspace_nodes(struct lockspace *ls, int option, int *node_count,
+                        struct dlmc_node **nodes_out)
+{
+	struct change *cg;
+	struct node *n;
+	struct dlmc_node *nodes = NULL, *nodep;
+	struct member *memb;
+	int count = 0;
+
+	if (option == DLMC_NODES_ALL) {
+		if (!list_empty(&ls->changes))
+			cg = list_first_entry(&ls->changes, struct change,list);
+		else
+			cg = ls->started_change;
+
+		list_for_each_entry(n, &ls->node_history, list)
+			count++;
+
+	} else if (option == DLMC_NODES_MEMBERS) {
+		if (!ls->started_change)
+			goto out;
+		cg = ls->started_change;
+		count = cg->member_count;
+
+	} else if (option == DLMC_NODES_NEXT) {
+		if (list_empty(&ls->changes))
+			goto out;
+		cg = list_first_entry(&ls->changes, struct change, list);
+		count = cg->member_count;
+	} else
+		goto out;
+
+	nodes = malloc(count * sizeof(struct dlmc_node));
+	if (!nodes)
+		return -ENOMEM;
+	memset(nodes, 0, count * sizeof(struct dlmc_node));
+	nodep = nodes;
+
+	if (option == DLMC_NODES_ALL) {
+		list_for_each_entry(n, &ls->node_history, list)
+			_set_node_info(ls, cg, n->nodeid, nodep++);
+	} else {
+		list_for_each_entry(memb, &cg->members, list)
+			_set_node_info(ls, cg, memb->nodeid, nodep++);
+	}
+ out:
+	*node_count = count;
+	*nodes_out = nodes;
+	return 0;
+}
+
