@@ -1,13 +1,23 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) 2005-2007 Red Hat, Inc.  All rights reserved.
+**
+**  This copyrighted material is made available to anyone wishing to use,
+**  modify, copy, or redistribute it subject to the terms and conditions
+**  of the GNU General Public License v.2.
+**
+*******************************************************************************
+******************************************************************************/
+
 #include <libcman.h>
 #include "dlm_daemon.h"
 
-int			our_nodeid;
 static cman_handle_t	ch;
 static cman_node_t      old_nodes[MAX_NODES];
 static int              old_node_count;
 static cman_node_t      cman_nodes[MAX_NODES];
 static int              cman_node_count;
-extern struct list_head lockspaces;
 
 static int is_member(cman_node_t *node_list, int count, int nodeid)
 {
@@ -61,6 +71,8 @@ static void statechange(void)
 	int num_addrs;
 	struct cman_node_address *addrptr = addrs;
 
+	cman_quorate = cman_is_quorate(ch);
+
 	old_node_count = cman_node_count;
 	memcpy(&old_nodes, &cman_nodes, sizeof(old_nodes));
 
@@ -135,7 +147,7 @@ static void member_callback(cman_handle_t h, void *private, int reason, int arg)
 	}
 }
 
-void process_member(int ci)
+void process_cman(int ci)
 {
 	int rv;
 
@@ -149,7 +161,7 @@ void process_member(int ci)
 	}
 }
 
-int setup_member(void)
+int setup_cman(void)
 {
 	cman_node_t node;
 	int rv, fd;
@@ -193,7 +205,8 @@ int setup_member(void)
 }
 
 /* Force re-read of cman nodes */
-void cman_statechange()
+void cman_statechange(void)
 {
 	statechange();
 }
+
