@@ -1,10 +1,18 @@
 #!/bin/bash
 #
+# Copyright 2003-2004, 2006-2007 Red Hat, Inc.
+#
 # Author(s):
 #     Hardy Merrill <hmerrill at redhat.com>
 #     Lon Hohberger <lhh at redhat.com>
 #     Michael Moon <Michael dot Moon at oracle.com>
 #
+# This program is Open Source software.  You may modify and/or redistribute
+# it persuant to the terms of the Open Software License version 2.1, which
+# is available from the following URL and is included herein by reference:
+#
+# 	http://opensource.org/licenses/osl-2.1.php
+#
 # chkconfig: 345 99 01
 # description: Service script for starting/stopping      \
 #	       Oracle(R) Database 10g on                 \
@@ -43,7 +51,6 @@ declare SCRIPTDIR="`dirname $0`"
 [ -n "$OCF_RESKEY_user" ] && ORACLE_USER=$OCF_RESKEY_user
 [ -n "$OCF_RESKEY_home" ] && ORACLE_HOME=$OCF_RESKEY_home
 [ -n "$OCF_RESKEY_name" ] && ORACLE_SID=$OCF_RESKEY_name
-[ -n "$OCF_RESKEY_listener_name" ] && ORACLE_LISTENER=$OCF_RESKEY_listener_name
 [ -n "$OCF_RESKEY_lockfile" ] && LOCKFILE=$OCF_RESKEY_lockfile
 [ -n "$OCF_RESKEY_type" ] && ORACLE_TYPE=$OCF_RESKEY_type
 [ -n "$OCF_RESKEY_vhost" ] && ORACLE_HOSTNAME=$OCF_RESKEY_vhost
@@ -83,7 +90,7 @@ declare SCRIPTDIR="`dirname $0`"
 #
 # 5. Type of Oracle Database.  Currently supported: 10g 10g-iAS(untested!)
 #
-[ -n "$ORACLE_TYPE" ] || ORACLE_TYPE="base-em"
+[ -n "$ORACLE_TYPE" ] || ORACLE_TYPE=10g
 
 #
 # 6. Oracle virtual hostname.  This is the hostname you gave Oracle during
@@ -143,21 +150,7 @@ meta_data()
 	    <content type="string"/>
         </parameter>
 
-        <parameter name="listener_name" unique="1">
-	    <longdesc lang="en">
-		Oracle Listener Instance Name.  If you have multiple 
-		instances of Oracle running, it may be necessary to 
-		have multiple listeners on the same machine with
-		different names.
-	    </longdesc>
-            <shortdesc lang="en">
-		Oracle Listener Instance Name
-            </shortdesc>
-	    <content type="string"/>
-        </parameter>
-
-
-        <parameter name="user" required="1">
+        <parameter name="user" unique="1" required="1">
 	    <longdesc lang="en">
 		Oracle user name.  This is the user name of the Oracle
 		user which the Oracle AS instance runs as.
@@ -168,7 +161,7 @@ meta_data()
 	    <content type="string"/>
         </parameter>
 
-        <parameter name="home" required="1">
+        <parameter name="home" unique="1" required="1">
 	    <longdesc lang="en">
 		This is the Oracle (application, not user) home directory.
 		This is configured when you install Oracle.
@@ -179,13 +172,11 @@ meta_data()
 	    <content type="string"/>
         </parameter>
 
-        <parameter name="type" required="0">
+        <parameter name="type" required="1">
 	    <longdesc lang="en">
-		This is the Oracle installation type:
-		base - Database Instance and Listener only
-		base-em (or 10g) - Database, Listener, Enterprise Manager,
-				   and iSQL*Plus
-		ias (or 10g-ias) - Internet Application Server (Infrastructure)
+		This is the Oracle installation type.
+		Only "10g" and "10g-ias" are supported, and 10g-ias is
+		untested.
 	    </longdesc>
             <shortdesc lang="en">
 		Oracle Installation Type
@@ -227,7 +218,7 @@ meta_data()
 	<action name="monitor" depth="20" timeout="90" interval="10m"/>
 
 	<action name="meta-data" timeout="5"/>
-	<action name="validate-all" timeout="5"/>
+	<action name="verify-all" timeout="5"/>
     </actions>
 
     <special tag="rgmanager">
@@ -299,7 +290,7 @@ start_db()
 	grep -q "failure" $logfile
 	if [ $? -eq 0 ]; then
 		rm -f $tmpfile
-	echo "ORACLE_SID Incorrectly set?"
+		echo "ORACLE_SID Incorrectly set?"
 		echo "See $logfile for more information."
 		return 1
 	fi
@@ -481,9 +472,8 @@ get_lsnr_status()
 	#
 	for (( i=$RESTART_RETRIES ; i; i-- )) ; do
 
-		action "Restarting Oracle listener:" lsnrctl start \
-					$ORACLE_LISTENER
-		lsnrctl status $ORACLE_LISTENER >& /dev/null
+		action "Restarting Oracle listener:" lsnrctl start
+		lsnrctl status >& /dev/null
 		if [ $? == 0 ] ; then
 			break # Listener was (re)started and is running fine
 		fi
@@ -685,16 +675,7 @@ validation_checks()
 	#
 	# If we don't know the type, we're done
 	#
-	if [ "$ORACLE_TYPE" = "base" ]; then
-		# Other names for base
-		ORACLE_TYPE="base"
-	elif [ "$ORACLE_TYPE" = "10g" ] || [ "$ORACLE_TYPE" = "base-em" ]; then
-		ORACLE_TYPE="base-em"
-	elif [ "$ORACLE_TYPE" = "10g-ias" ] || [ "$ORACLE_TYPE" = "ias" ]; then
-		ORACLE_TYPE="ias"
-	else
-		oops ORACLE_TYPE
-	fi
+	[ "$ORACLE_TYPE" = "10g" ] || [ "$ORACLE_TYPE" = "10g-ias" ] || oops ORACLE_TYPE
 
 	#
 	# If the hostname is zero-length, fix it
@@ -736,12 +717,12 @@ validation_checks()
 start_oracle()
 {
 	faction "Starting Oracle Database:" start_db || return 1
-	action "Starting Oracle Listener:" lsnrctl start $ORACLE_LISTENER || return 1
+	action "Starting Oracle Listener:" lsnrctl start || return 1
 
-	if [ "$ORACLE_TYPE" = "base-em" ]; then
+	if [ "$ORACLE_TYPE" = "10g" ]; then
 		action "Starting iSQL*Plus:" isqlplusctl start || return 1
 		action "Starting Oracle EM DB Console:" emctl start dbconsole || return 1
-	elif [ "$ORACLE_TYPE" = "ias" ]; then
+	elif [ "$ORACLE_TYPE" = "10g-ias" ]; then
 		action "Starting Oracle EM:" emctl start em || return 1
 		action "Starting iAS Infrastructure:" opmnctl startall || return 1
 	fi
@@ -764,16 +745,16 @@ stop_oracle()
 		return 0
 	fi
 
-	if [ "$ORACLE_TYPE" = "base-em" ]; then
+	if [ "$ORACLE_TYPE" = "10g" ]; then
 		action "Stopping Oracle EM DB Console:" emctl stop dbconsole || return 1
 		action "Stopping iSQL*Plus:" isqlplusctl stop || return 1
-	elif [ "$ORACLE_TYPE" = "ias" ]; then
+	elif [ "$ORACLE_TYPE" = "10g-ias" ]; then
 		action "Stopping iAS Infrastructure:" opmnctl stopall || return 1
 		action "Stopping Oracle EM:" emctl stop em || return 1
 	fi
 
 	faction "Stopping Oracle Database:" stop_db || return 1
-	action "Stopping Oracle Listener:" lsnrctl stop $ORACLE_LISTENER
+	action "Stopping Oracle Listener:" lsnrctl stop
 	faction "Waiting for all Oracle processes to exit:" exit_idle 
 
 	if [ $? -ne 0 ]; then
@@ -828,12 +809,12 @@ status_oracle()
 	update_status $? $last
 	last=$?
 	
-	if [ "$ORACLE_TYPE" = "base-em" ]; then
+	if [ "$ORACLE_TYPE" = "10g" ]; then
 		# XXX Add isqlplus status check?!
 		emctl status dbconsole 2>&1 | grep "is running"
 		update_status $? $last
 		last=$?
-	elif [ "$ORACLE_TYPE" = "ias" ]; then
+	elif [ "$ORACLE_TYPE" = "10g-ias" ]; then
 		# Check & report opmn / opmn-managed process status
 		get_opmn_status $subsys_lock $depth
 		update_status $? $last
